--[[
	A CS2D Lua script to allow for more than 16 maps in the mapvote (vanilla)
	
	This script uses a huge list of maps (more than 16) in sys/mapcycleExtended.cfg  and picks 16 random maps and puts them into sys/mapcycle.cfg.
	
	Those maps are not part of:
		- the recent mapcycle
		- current or new map
		
	WARNING: 
		Using this script will delete your original sys/mapcycle.cfg! Back it up before using this script!
	
]]


if not mce then mce = {} end

--[[
	You can change this number to limit the amount of maps in the mapcycle.cfg
	Must be between 1 and 16
]]
mce.maxMaps = 12;




mce.fileExtendedPath = "sys/mapcycleExtended.cfg";
mce.filePath = "sys/mapcycle.cfg";
mce.fileRecentPath = "sys/lua/mapcycleData.cfg";


--[[
function mce.debug(txt)
	print("\169255255255MapCycleExtended: "..txt)
end
]]--

addhook("mapchange","mce.onMapchange")
function mce.onMapchange(newmap)
	math.randomseed(os.time())
	if (mce.maxMaps > 16)  then
		mce.maxMaps = 16;
	end
	local lines = {};
	local linesCount = 0;
	local commentedLine;
	
	for line in io.lines(mce.fileExtendedPath) do 
		if not line:match("^%s*$") then
    
			line = string.gsub(line, "^%s+", ""); -- remove front spaces
			commentedLine = string.match(line, "^//");
			if (commentedLine == nil) then -- no front '//'s
				linesCount = linesCount + 1;
				lines[linesCount] = {mce.getMapName(line), line};
			end
		end
	end
	
	local mapAmount
	if (linesCount > mce.maxMaps) then
		mapAmount =  mce.maxMaps;
	else
		mapAmount = linesCount;
	end
	
	local recentMaps = {}
	for line in io.lines(mce.fileRecentPath) do
		recentMaps[#recentMaps+1] = line;
	end
	
	local remainingMaps = mapAmount;
	local choice;
	local selectedMaps = {};
	local isKnown = false;
	local file;
	file = io.open(mce.filePath, "w+")
	io.output(file)
	
	local race = 1000;
	
	while (remainingMaps > 0)  do
		choice = lines[math.random(linesCount)];
		
		--mce.debug("Choice: "..choice[1].. "// remainingMaps: "..remainingMaps)
		
		-- figure out, wether new random choice is already taken
		if (choice[1] == newmap or 
			choice[1] == map("name") or 
			mce.isContaining(choice, selectedMaps) or 
			mce.isContaining(choice[1], recentMaps)
			) then
			isKnown = true;
			--mce.debug("Choice is already known :( ")
		end
		
		-- add choice if its new
		if not isKnown then 
			remainingMaps = remainingMaps - 1;
			selectedMaps[#selectedMaps+1] = choice;
			--mce.debug("Added "..choice[1])
			io.write(choice[2].."\n");
		end
		
		isKnown = false;
		
		race = race -1
		if (race <= 0) then
			--mce.debug("Race condition! Cant find next new map")
			break;
		end
	end
	io.close(file)
	
	file = io.open(mce.fileRecentPath, "w+")
	io.output(file)
	if (mapAmount <= linesCount) then
		-- Remember last map vote list
		for _,v in pairs(selectedMaps) do
			io.write(v[1].."\n")
		end
		
	end
	io.close(file);
end


function mce.isContaining(element, array)
	for _,v in ipairs(array) do
		if (element == v) then
			return true;
		end
	end
	return false;
end

function mce.getMapName(line)
	local startBracket = string.find(line, "{");
	if (startBracket ~= nil) then
		line = line:sub(1, startBracket-1);
	end
	return string.gsub(line,"%s+", "");
end



